using System.Collections.Generic;
using static ChineseChess.ChessCore;
using Vector2I = Vector.Vector2I;

namespace ChineseChess;
public class CCGeneral : CCPiece {
    public CCGeneral(CCBoard board, TurnsSideType turnsSide = TurnsSideType.Red, Vector2I? pos = null)
    : base(board, turnsSide, name : "General", pos) {
        if (turnsSide == TurnsSideType.Red) {
            CNName = "帅";
        } else {
            CNName = "将";
        }
    }

    public override List<Vector2I> CanMoveAllPosSelf() {
        List<Vector2I> list = new() {
            new(1, 0),
            new(-1, 0),
            new(0, 1),
            new(0, -1),
        };

        // 移除不符合条件的元素
        list.RemoveAll(item =>
            localPos.X + item.X > 1 || localPos.X + item.X < -1 ||
            localPos.Y + item.Y > 2 || localPos.Y + item.Y < 0
            || GetRecursivePieceLocal(localPos + item, new(0, 1)) is CCGeneral);
        return list;
    }
}

public class CCAdvisor : CCPiece {
    public CCAdvisor(CCBoard board, TurnsSideType turnsSide = TurnsSideType.Red, Vector2I? pos = null)
    : base(board, turnsSide, name : "Advisor", pos) {
        if (turnsSide == TurnsSideType.Red) {
            CNName = "仕";
        } else {
            CNName = "士";
        }
    }

    public override List<Vector2I> CanMoveAllPosSelf() {
        List<Vector2I> list = new() {
            new(1, 1),
            new(-1, 1),
            new(1, -1),
            new(-1, -1),
        };

        // 移除不符合条件的元素
        list.RemoveAll(item =>
            localPos.X + item.X > 1 || localPos.X + item.X < -1 ||
            localPos.Y + item.Y > 2 || localPos.Y + item.Y < 0);

        return list;
    }
}

public class CCElephant : CCPiece {
    public CCElephant(CCBoard board, TurnsSideType turnsSide = TurnsSideType.Red, Vector2I? pos = null)
    : base(board, turnsSide, name : "Bishop", pos) {
        if (turnsSide == TurnsSideType.Red) {
            CNName = "相";
        } else {
            CNName = "象";
        }
    }

    public override List<Vector2I> CanMoveAllPosSelf() {
        List<Vector2I> list = new() {
            new(2, 2),
            new(-2, 2),
            new(2, -2),
            new(-2, -2),
        };

        list.RemoveAll(item => IsPosOutOfRangeLocal(localPos + item));
        // 移除不符合条件的元素
        list.RemoveAll(item => localPos.Y + item.Y < 0 || localPos.Y + item.Y > 4 ||
            GetCCPieceLocal(new(localPos.X + item.X / 2, localPos.Y + item.Y / 2)) is not null);

        return list;
    }
}

public class CCHorse : CCPiece {
    public CCHorse(CCBoard board, TurnsSideType turnsSide = TurnsSideType.Red, Vector2I? pos = null)
    : base(board, turnsSide, name : "Horse", pos) {
        if (turnsSide == TurnsSideType.Red) {
            CNName = "馬";
        } else {
            CNName = "马";
        }
    }

    public override List<Vector2I> CanMoveAllPosSelf() {
        List<Vector2I> list = new () {
            new Vector2I(1, 2),
            new Vector2I(1, -2),
            new Vector2I(2, 1),
            new Vector2I(2, -1),
            new Vector2I(-1, -2),
            new Vector2I(-1, 2),
            new Vector2I(-2, -1),
            new Vector2I(-2, 1),
        };

        list.RemoveAll(item => IsPosOutOfRangeLocal(localPos + item));
        list.RemoveAll(item => {
            Vector2I pos = new(localPos);
            if (item.X == 2) {
                pos.X += 1;
            } else if (item.X == -2) {
                pos.X -= 1;
            } else if (item.Y == 2) {
                pos.Y += 1;
            } else if (item.Y == -2) {
                pos.Y -= 1;
            }
            return GetCCPieceLocal(pos) is not null;
        });
        return list;
    }
}

public class CCChariot : CCPiece {
    public CCChariot(CCBoard board, TurnsSideType turnsSide = TurnsSideType.Red, Vector2I? pos = null)
    : base(board, turnsSide, name : "Chariot", pos) {
        if (turnsSide == TurnsSideType.Red) {
            CNName = "車";
        } else {
            CNName = "车";
        }
    }

    public override List<Vector2I> CanMoveAllPosSelf() {
        List<Vector2I> list = new ();

        void func(Vector2I added) {
            Vector2I ptr = new(localPos);
            while (true) {
                ptr += added;
                if (IsPosOutOfRangeLocal(ptr)) {
                    break;
                }
                if (GetCCPieceLocal(ptr) != null) {
                    list.Add(ptr - localPos);
                    break;
                }
                list.Add(ptr - localPos);
            }
        }

        func(Vector2I.Up);
        func(Vector2I.Down);
        func(Vector2I.Left);
        func(Vector2I.Right);
        return list;
    }
}

public class CCCannon : CCPiece {
    public CCCannon(CCBoard board, TurnsSideType turnsSide = TurnsSideType.Red, Vector2I? pos = null)
    : base(board, turnsSide, name : "Cannon", pos) {
        if (turnsSide == TurnsSideType.Red) {
            CNName = "砲";
        } else {
            CNName = "炮";
        }
    }

    public override List<Vector2I> CanMoveAllPosSelf() {
        List<Vector2I> list = new ();

        void func(Vector2I added) {
            Vector2I ptr = new(localPos);
            bool flag = true;
            while (true) {
                ptr += added;
                if (IsPosOutOfRangeLocal(ptr)) {
                    break;
                }
                if (GetCCPieceLocal(ptr) != null) {
                    if (flag) {
                        flag = false;
                    } else {
                        list.Add(ptr - localPos);
                        break;
                    }
                }
                if (flag) list.Add(ptr - localPos);
            }
        }

        func(Vector2I.Up);
        func(Vector2I.Down);
        func(Vector2I.Left);
        func(Vector2I.Right);
        return list;
    }
}

public class CCPawn : CCPiece {
    public CCPawn(CCBoard board, TurnsSideType turnsSide = TurnsSideType.Red, Vector2I? pos = null)
    : base(board, turnsSide, name : "Pawn", pos) {
        if (turnsSide == TurnsSideType.Red) {
            CNName = "兵";
        } else {
            CNName = "卒";
       }
    }

    public override List<Vector2I> CanMoveAllPosSelf() {
        List<Vector2I> list = new () {
            new(0, 1),
            new(1, 0),
            new(-1, 0),
        };

        list.RemoveAll(item => IsPosOutOfRangeLocal(localPos + item));
        list.RemoveAll(item => localPos.Y <= 4 && item != new Vector2I(0, 1));
        return list;
    }
}

// 帅/将 (General) - 代表双方的最高统帅。
// 子类名：ChessGeneral

// 仕/士 (Advisor) - 保护帅/将的近身侍卫。
// 子类名：ChessAdvisor

// 相/象 (Elephant) - 行动受限，走田字格，不能过河。
// 子类名：ChessElephant

// 車/车 (Chariot) - 横竖移动，威力巨大。
// 子类名：ChessChariot

// 馬/马 (Horse) - 走日字形，跳跃式移动。
// 子类名：ChessHorse

// 砲/炮 (Cannon) - 需要隔子才能吃子，直线移动。
// 子类名：ChessCannon

// 兵/卒 (Pawn) - 最基础的棋子，过河后可横移。
// 子类名：ChessPawn